package org.bench4q.agent.scenario.node;

import java.util.ArrayList;
import java.util.Stack;

import org.bench4q.agent.scenario.instruction.Instruction;
import org.bench4q.agent.scenario.instruction.TestInstruction;
import org.bench4q.share.models.agent.scriptrecord.BehaviorModel;
import org.bench4q.share.models.agent.scriptrecord.ConditionBehaviorModel;
import org.bench4q.share.models.agent.scriptrecord.IfBehaviorModel;

/**
 * 
 * @author coderfengyun
 *
 */
public abstract class Behavior {
	public static final String CONTROL_BEHAVIOR = "CONTROLBEHAVIOR";
	public static final String USER_BEHAVIOR = "USERBEHAVIOR";
	public static final String TIMER_BEHAVIOR = "TIMERBEHAVIOR";
	public static final String TEST_BEHAVIOR = "TESTBEHAVIOR";

	/**
	 * No matter which one to inherit from this class, it should call
	 * baseCompile in its own compile function.
	 * 
	 * This function has side effect, the result is both code and conditions
	 */

	public abstract void compile(ArrayList<Instruction> code,
			Stack<TestInstruction> conditions);

	private int id;

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public static Behavior buildWith(BehaviorModel behaviorModel) {
		Behavior behavior = null;
		if (behaviorModel.getType().equalsIgnoreCase(TIMER_BEHAVIOR)) {
			behavior = new TimerBehavior(behaviorModel);
		} else if (behaviorModel.getType().equalsIgnoreCase(USER_BEHAVIOR)) {
			behavior = new UserBehavior(behaviorModel);
		} else if (behaviorModel.getType().equalsIgnoreCase(CONTROL_BEHAVIOR)) {
			behavior = new ControlBehavior(behaviorModel);
		} else if (behaviorModel.getType().equalsIgnoreCase(TEST_BEHAVIOR)) {
			// TODO: refactor this
			if (behaviorModel instanceof IfBehaviorModel) {
				behavior = new IfBehavior((IfBehaviorModel) behaviorModel);
			} else if (behaviorModel instanceof ConditionBehaviorModel) {
				behavior = new ConditionBehavior(
						(ConditionBehaviorModel) behaviorModel);
			} else {
				throw new RuntimeException("Error type: TestInstruction"
						+ behaviorModel.getClass().getName());
			}
		} else {
			throw new IllegalArgumentException(
					"The input BehaviorModel's type is not proper");
		}
		return behavior;
	}

}
