package org.bench4q.master.domain.valueobject.datastatistics.impl;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.bench4q.share.models.agent.statistics.BehaviorBriefModel;
import org.bench4q.share.models.agent.statistics.BehaviorStatusCodeResultModel;
import org.bench4q.share.models.agent.statistics.DataStatisticsModel;
import org.bench4q.share.models.master.statistics.ScriptBehaviorBriefModel;
import org.bench4q.share.models.master.statistics.ScriptBehaviorStatusCodeModel;

public class BehaviorBriefStatistics extends ScriptStatistics {
	private final int behaviorId;
	private final String behaviorUrl;
	private long successCountFromBegin;
	private long totalCountFromBegin;
	private volatile Map<Integer, BehaviorStatusCodeStatistics> detailStatisticsMap;

	public BehaviorBriefStatistics(int behaviorId, String behaviorUrl) {
		this.behaviorId = behaviorId;
		this.behaviorUrl = behaviorUrl;
		this.detailStatisticsMap = new ConcurrentHashMap<Integer, BehaviorStatusCodeStatistics>();
		resetTempraryFields();
	}

	/**
	 * Now, for totalCount, i just collect all result from agent, and add their
	 * countFromBegin together. But there will be a error there. if an agent
	 * doesn't give back its result, the the countFromBegin maybe less that last
	 * one.
	 */
	private void resetTempraryFields() {
		this.successCountFromBegin = 0;
		this.totalCountFromBegin = 0;
	}

	@Override
	public void add(DataStatisticsModel dataUnit) {
		if (!(dataUnit instanceof BehaviorBriefModel)) {
			return;
		}
		BehaviorBriefModel behaviorBriefModel = (BehaviorBriefModel) dataUnit;
		this.totalCountFromBegin += behaviorBriefModel.getTotalCount();
		this.successCountFromBegin += behaviorBriefModel.getSuccessfulCount();
		for (BehaviorStatusCodeResultModel newOne : behaviorBriefModel
				.getDetailStatusCodeResultModels()) {
			if (!this.detailStatisticsMap.containsKey(newOne.getStatusCode())) {
				detailStatisticsMap
						.put(newOne.getStatusCode(),
								new BehaviorStatusCodeStatistics(
										this.behaviorId,
										newOne.getStatusCode(), newOne
												.getContentType()));
			}
			detailStatisticsMap.get(newOne.getStatusCode()).add(newOne);
		}
	}

	@Override
	public Object getStatistics() {
		ScriptBehaviorBriefModel result = new ScriptBehaviorBriefModel();
		result.setBehaviorId(this.behaviorId);
		result.setBehaviorUrl(this.behaviorUrl);
		result.setSuccessCountFromBegin(this.successCountFromBegin);
		result.setTotalCountFromBegin(this.totalCountFromBegin);
		for (BehaviorStatusCodeStatistics item : this.detailStatisticsMap
				.values()) {
			result.getDetailStatusList().add(
					(ScriptBehaviorStatusCodeModel) item.getStatistics());
		}
		return result;
	}

}
